#ifndef LEU_UTIL_H
#define LEU_UTIL_H

#include <stdio.h>
#include <iostream>
#include <cstring>
#include "LEU_LIB.h"

#define MYUSB_MAXPACKET 4096
#define MYUSB_ENDFLAG1 0x1
#define MYUSB_ENDFLAG2 0x2
#define MYUSB_PREFLAG  0x4

/// USB transaction data struct ///
typedef struct{
	float time;
	int speed;
	int split;
	int event_id;
	int token_pid;
	uint8_t addr;
	uint8_t endp;
	int data_pid;
	int hand_pid;
	int data_length;
	uint16_t frame;
	float busreset_time;
	int trig_no;
	float vbus_V;
	float vbus_I;
	uint8_t data[MYUSB_MAXPACKET];
	int endflag;	
} USBTrans;


/// for display ///
#define MYDISP_NCHR 8
#define MYDISP_DATALEN (MYDISP_NCHR * 3 + 4)

#define FLAG_DISP_SOF 0
#define FLAG_DISP_PING 1

#define DISP_PID 0x1
#define DISP_VALUE 0x2
#define DISP_ADDR 0x4
#define DISP_ENDP 0x8
#define DISP_DATA 0x10
#define DISP_HAND 0x20
#define DISP_SPD 0x40
#define DISP_NTIMES 0x80
#define DISP_NOLINE 0x100
#define DISP_DATA_ID 0x200
#define DISP_DATA_LEN 0x400

#define NAME_BUSRESET 0
#define NAME_SUSPEND 1
#define NAME_DISCONNECT 2
#define NAME_DEVCHIRP 3
#define NAME_HUBCHIRP 4
#define NAME_VBUS 5
#define NAME_TRIG 6
#define NAME_TRIGEX 7
#define NAME_RSV 8
#define NAME_PING 9
#define NAME_SOF 10
#define NAME_SPLIT 11
#define NAME_IN 12
#define NAME_OUT 13
#define NAME_SETUP 14

#define NAME_LSPD 15
#define NAME_FSPD 16
#define NAME_HSPD 17

#define NAME_DATA0 18
#define NAME_DATA1 19
#define NAME_DATA2 20
#define NAME_MDATA 21

#define NAME_NAK 22
#define NAME_ACK 23
#define NAME_NYET 24
#define NAME_STALL 25
#define NAME_ERR 26

#define NAME_NONE 27

const char NAME_DISP[28][10]={
	"BUSRESET",
	"SUSPEND",
	"DISCONCT.",
	"DEVCHIRP",
	"HUBCHIRP",
	"VBUS",
	"TRIG",
	"TRIGEX",
	"RESERVE",
	"PING",
	"SOF",
	"SPLIT",
	"IN",
	"OUT",
	"SETUP",
	"LOW",
	"FULL",
	"HIGH",
	"DATA0",
	"DATA1",
	"DATA2",
	"MDATA",
	"NAK",
	"ACK",
	"NYET",
	"STALL",
	"ERR",
	"NONE"
};

/// Transaction reconstruction class ///
class MyTrans{
 public:
	MyTrans();
	~MyTrans();

	int dispflag;
	float last_time;
	int Append(LEU_data &ldata); // append a packet to transaction
	const char * GetDispStr(int did); 	// Get disply strings
	int NextTrans(); // move to the next stored transaction
	
 private:
	int transflag;
	USBTrans trans; // current transaction (may be incomplete)
	USBTrans trans_prev; // previous transaction
	USBTrans trans_last; // last valid transaction decoded
	
	int nSameTrans;
	int SameTrans(USBTrans &t1, USBTrans &t2);
	void CopyTrans(USBTrans &t1, USBTrans &t2);

	int GetStrings(int tID);
	int NewTrans();
	void ClearAll();
	void Clear(USBTrans &t_trans);

	// packet analysis
	int IsNewTrans(USBTrans trans, int pid);
	int IsEndTrans(int pid);
	int IsToken(int pid);
	int IsHandShake(int pid);

	//display string buffer
	int name_id;
	int spd_id;
	int hand_id;
	int data_id;
	char buf_value[50];
	char buf_data[3 * MYDISP_NCHR +4];
	char buf_tmp[20];
};

void GetErrStatus();

#endif
